import os
import contextlib
from spotipy.model import Config
import albumentations as alb

with open(os.devnull, "w") as f, contextlib.redirect_stdout(f):
    config_unet = Config(
        n_channel_in=1,
        backbone="unet",
        mode="bce",
        unet_n_depth=3,
        unet_pool=4,
        unet_n_filter_base=64,
        spot_weight=40,
        multiscale=False,
        train_learning_rate=3e-4,
        train_foreground_prob=1,
        train_batch_norm=False,
        train_multiscale_loss_decay_exponent=1,
        train_patch_size=(512, 512),
        spot_weight_decay=0.5,
        train_batch_size=2,
    )
    config_multiscale = Config(
        n_channel_in=1,
        backbone="unet",
        mode="mae",
        unet_n_depth=3,
        unet_pool=4,
        unet_n_filter_base=64,
        spot_weight=40,
        multiscale=True,
        train_learning_rate=3e-4,
        train_foreground_prob=1,
        train_batch_norm=False,
        train_multiscale_loss_decay_exponent=1,
        train_patch_size=(512, 512),
        spot_weight_decay=0.5,
        train_batch_size=2,
    )

transforms = alb.Compose(
    [
        alb.ShiftScaleRotate(scale_limit=0.0),
        alb.Flip(),
        alb.RandomBrightnessContrast(always_apply=True),
        alb.RandomGamma(),
    ]
)


from pathlib import Path
from typing import List

import numpy as np
import tifffile as tf
from attrs import define, field, validators
from cenfind.core.log import get_logger

logger = get_logger(__name__)


def path_exists(instance, attribute, value):
    if not value.exists():
        raise FileNotFoundError(f"Invalid path ({value})")


def has_projections(instance, attribute, value):
    projections = value / 'projections'
    if not projections.is_dir():
        raise FileNotFoundError(f"Invalid path for Projections: ({value}).")


def is_tif(instance, attribute, value):
    if value.suffix != '.tif':
        raise ValueError(f'Not a tif file ({value})')


@define
class Field:
    path: Path = field(validator=[validators.instance_of(Path), path_exists, is_tif])

    @property
    def name(self) -> str:
        """
        Return the field name as a string without the extension
        """
        return self.path.stem

    @property
    def data(self) -> np.ndarray:
        """
        Load the data into a numpy array
        """
        return tf.imread(str(self.path))


@define
class Dataset:
    """
    Represent a dataset structure
    """
    path: Path = field(validator=[
        validators.instance_of(Path),
        path_exists,
        has_projections])

    @property
    def logs(self):
        return self.path / 'logs'

    @property
    def predictions(self):
        return self.path / 'predictions'

    @property
    def visualisation(self):
        return self.path / 'visualisation'

    @property
    def statistics(self):
        return self.path / 'statistics'

    @property
    def nuclei(self):
        return self.predictions / 'nuclei'

    @property
    def centrioles(self):
        return self.predictions / 'centrioles'

    @property
    def cilia(self):
        return self.predictions / 'cilia'

    @property
    def assignment(self):
        return self.predictions / 'assignment'

    def setup(self):
        self.visualisation.mkdir(exist_ok=True)
        self.statistics.mkdir(exist_ok=True)
        self.predictions.mkdir(exist_ok=True)
        self.nuclei.mkdir(exist_ok=True)
        self.centrioles.mkdir(exist_ok=True)
        self.cilia.mkdir(exist_ok=True)
        self.assignment.mkdir(exist_ok=True)

    @property
    def fields(self) -> List[Field]:
        """
        Return a list of Fields found in projections.
        """
        path = self.path / 'projections'
        result = [Field(path) for path in path.iterdir() if
                  (path.suffix == '.tif') and not path.name.startswith('.')]
        if len(result) == 0:
            raise ValueError(f'No field found in {path}')
        return result



import argparse
from pathlib import Path

import tifffile as tf

from cenfind.core.data import Dataset
from cenfind.core.visualisation import visualisation
from cenfind.core.log import get_logger


def register_parser(parent_subparsers):
    parser = parent_subparsers.add_parser(
        "predict",
        help="Predict centrioles on new dataset and save under visualisation/runs/<model_name>",
    )
    parser.add_argument("dataset", type=Path, help="Path to the dataset folder")
    parser.add_argument("model", type=Path, help="Path to the model")
    parser.add_argument(
        "--channel_nuclei", type=int, required=True, help="Index of nuclei channel"
    )

    return parser


def run(args):
    dataset = Dataset(args.dataset)
    dataset.setup()
    logger = get_logger(__name__, file=dataset.logs / 'cenfind.log')

    model_predictions = dataset.visualisation / "runs"
    model_predictions.mkdir(exist_ok=True)

    model_run = model_predictions / args.model.name
    model_run.mkdir(exist_ok=True)

    from cenfind.core.detectors import extract_foci, extract_nuclei
    pairs = dataset.splits()
    for field, channel in pairs["test"]:
        nuclei = extract_nuclei(field=field, channel=args.channel_nuclei)
        foci = extract_foci(field, channel, args.model, prob_threshold=0.5)
        logger.info(
            "Writing visualisation for field: %s, channel: %s, %s foci detected"
            % (field.name, channel, len(foci))
        )
        vis = visualisation(field, channel_centrioles=channel, channel_nuclei=args.channel_nuclei, nuclei=nuclei)
        tf.imwrite(model_run / f"{field.name}_C{channel}_pred.png", vis)


if __name__ == "__main__":
    args = argparse.Namespace(dataset=Path('data/dataset_test'),
                              model=Path('models/master'),
                              channel_nuclei=0,
                              factor=256)
    run(args)



from typing import Tuple, List

import cv2
import numpy as np
import pandas as pd
from attrs import define
from ortools.linear_solver import pywraplp

from cenfind.core.log import get_logger
from cenfind.core.structures import Centriole, Nucleus

logger = get_logger(__name__)


def flag(is_full: bool) -> tuple:
    """
    Helper function to change colour of contour.
    Parameters
    ----------
    is_full

    Returns
    -------

    """
    return (0, 255, 0) if is_full else (0, 0, 255)


def signed_distance(focus: Centriole, nucleus: Nucleus) -> float:
    """
    Wrapper for the opencv PolygonTest
    """

    result = cv2.pointPolygonTest(nucleus.contour, focus.centre_xy, measureDist=True)
    return result


@define
class Assigner:
    centrioles: List[Centriole]
    nuclei: List[Nucleus]
    vicinity: float = 0
    assignment: np.ndarray = None

    def _compute(self, vicinity: float = 0) -> np.ndarray:
        num_nuclei = len(self.nuclei)
        num_centrioles = len(self.centrioles)

        costs = {}
        for i in range(num_nuclei):
            for j in range(num_centrioles):
                dist = signed_distance(self.centrioles[j], self.nuclei[i])
                costs[i, j] = dist + vicinity
        solver = pywraplp.Solver.CreateSolver("SCIP")

        x = {}
        for i in range(num_nuclei):
            for j in range(num_centrioles):
                x[i, j] = solver.IntVar(0, 1, "")

        for j in range(num_centrioles):
            solver.Add(solver.Sum([x[i, j] for i in range(num_nuclei)]) <= 1)

        # Objective
        objective_terms = []
        for i in range(num_nuclei):
            for j in range(num_centrioles):
                objective_terms.append(costs[i, j] * x[i, j])
        solver.Maximize(solver.Sum(objective_terms))

        # Solve
        status = solver.Solve()
        if status != pywraplp.Solver.OPTIMAL and status != pywraplp.Solver.FEASIBLE:
            raise ValueError("No solution found.")

        result = np.zeros([num_nuclei, num_centrioles], dtype=bool)
        for i in range(num_nuclei):
            for j in range(num_centrioles):
                if x[i, j].solution_value() > 0:
                    result[i, j] = True

        return result

    def score_nuclei(self, field_name: str, channel: int) -> pd.DataFrame:
        if self.assignment is None:
            self.assignment = self._compute(self.vicinity)

        result = self.assignment.sum(axis=1)
        result = list(zip(self.nuclei, result))

        result = pd.DataFrame(list((n.index, s) for n, s in result))
        result.columns = ['nucleus', 'score']
        result["field"] = field_name
        result["channel"] = channel
        result = result.set_index(["field", "channel"])
        return result

    def assign_centrioles(self) -> List[Tuple[int, int]]:
        if self.assignment is None:
            self.assignment = self._compute(self.vicinity)

        result = []
        for c, centriole in enumerate(self.assignment.T):
            centriole_index = self.centrioles[c].index
            if centriole.max() == 0:
                result.append((centriole_index, -1))
            else:
                nucleus_matrix_index = np.where(centriole == 1)[0][0]
                nucleus_index = self.nuclei[nucleus_matrix_index].index
                result.append((centriole_index, nucleus_index))

        return result



import logging
import sys
from logging.handlers import TimedRotatingFileHandler
from pathlib import Path

FORMATTER = logging.Formatter("%(asctime)s — %(name)s — %(levelname)s — %(message)s")


def get_console_handler():
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setFormatter(FORMATTER)
    return console_handler


def get_null_handler():
    null_handler = logging.NullHandler()
    null_handler.setFormatter(FORMATTER)
    return null_handler


def get_file_handler(file):
    file_handler = TimedRotatingFileHandler(file, when='midnight')
    file_handler.setFormatter(FORMATTER)
    return file_handler


def get_logger(logger_name, console=None, file=None):
    logger = logging.getLogger(logger_name)
    logger.setLevel(logging.DEBUG)  # better to have too much log than not enough
    logger.addHandler(get_null_handler())
    if console:
        logger.addHandler(get_console_handler())
    if file:
        logger.addHandler(get_file_handler(file))

    # with this pattern, it's rarely necessary to propagate the error up to parent
    logger.propagate = False
    return logger


if __name__ == "__main__":
    print(Path(__file__).parents[3])



import argparse
import importlib
from argparse import ArgumentDefaultsHelpFormatter

from cenfind.core.log import get_logger

commands = [
    "score",
]

COMMANDS = [importlib.import_module("cenfind.cli." + c) for c in commands]


def add_command_default(parser):
    class default_command:
        def run(args):
            parser.print_help()
            return 2

    parser.set_defaults(__command__=default_command)


def add_command_subparsers(subparsers, commands, command_attribute="__command__"):
    for command in commands:
        subparser = command.register_parser(subparsers)

        if command_attribute:
            subparser.set_defaults(**{command_attribute: command})
            subparser.formatter_class = ArgumentDefaultsHelpFormatter

        if not subparser.description and command.__doc__:
            subparser.description = command.__doc__

        if not getattr(command, "run", None):
            add_command_default(subparser)


def make_parser():
    parser = argparse.ArgumentParser(
        prog="CENFIND",
        description="CENFIND: A deep-learning pipeline to score cells for centrioles",
    )

    add_command_default(parser)
    subparsers = parser.add_subparsers()
    add_command_subparsers(subparsers, COMMANDS)

    return parser


def run(argv):
    args = make_parser().parse_args(argv)

    logger = get_logger(__name__)

    try:
        return args.__command__.run(args)
    except RecursionError:
        logger.error("Maximum recursion depth reached.", exc_info=True)
        raise
    except FileNotFoundError:
        logger.error("File not found", exc_info=True)
        raise



import re
from pathlib import Path

import PIL
import labelbox
import numpy as np
import tifffile as tf
from dotenv import dotenv_values
from tqdm import tqdm

from cenfind.core.data import Dataset

config = dotenv_values(".env")


def download_centrioles(label):
    """
    Collect the positions for a given label.
    """
    foci_in_label = [lab for lab in label.annotations if lab.name == "Centriole"]
    positions = []

    for lab in foci_in_label:
        # the coordinates in labelbox are (x, y) and start
        # in the top left corner;
        # thus, they correspond to (col, row).
        x = int(lab.value.x)
        y = int(lab.value.y)
        positions.append((x, y))

    return np.array(positions, dtype=int)


def download_mask(label, object_type):
    """
    Collect the nucleus masks for the label.
    return a 16bit-numpy mask with each nucleus labelled by pixel value
    """
    ALLOWED_OBJECT_TYPES = ["Nucleus", "Cilium"]
    if object_type not in ALLOWED_OBJECT_TYPES:
        raise ValueError("object type must be of: %s" % ALLOWED_OBJECT_TYPES)

    objects_in_label = [lab for lab in label.annotations if lab.name == object_type]

    if len(objects_in_label) == 0:
        raise ValueError("Empty list; no nucleus")

    mask_shape = objects_in_label[0].value.mask.value.shape
    res = np.zeros(mask_shape[:2], dtype="uint16")
    object_id = 1

    for lab in objects_in_label:
        try:
            mask = lab.value.mask.value[:, :, 0]
            res += ((mask / 255) * object_id).astype("uint16")
            object_id += 1
        except PIL.UnidentifiedImageError as e:
            continue

    return res


def main():
    lb = labelbox.Client(api_key=config["LABELBOX_API_KEY"])
    project = lb.get_project(config["PROJECT_CENTRIOLES"])
    labels = project.label_generator()

    centrioles = True
    nuclei = False

    for label in tqdm(labels):
        ds = label.extra["Dataset Name"]

        if ds != "cilia":
            continue

        path_dataset = Path(f'data/{ds}')
        projection_suffix = ""
        dataset = Dataset(
            path_dataset, image_type=".tif", projection_suffix=projection_suffix
        )

        external_name = label.data.external_id
        extension = external_name.split(".")[-1]

        print("Processing %s / %s" % (path_dataset, external_name))

        if centrioles:
            annotation_name = re.sub(f".{extension}$", ".txt", external_name)
            dst_centrioles = dataset.path_annotations_centrioles / annotation_name
            try:
                positions = download_centrioles(label)
                np.savetxt(dst_centrioles, positions, delimiter=",", fmt="%u")
                print(
                    "Saving centriole positions of %s to %s"
                    % (external_name, dst_centrioles)
                )
            except FileExistsError:
                print(
                    "Skipping centriole positions for %s to %s"
                    % (external_name, dst_centrioles)
                )
                continue

        if nuclei:
            mask_name = re.sub(
                f"C\d\.{extension}$", f"{projection_suffix}_C0.tif", external_name
            )
            dst_nuclei = dataset.path_annotations_cells / mask_name
            try:
                mask = download_mask(label, "Nucleus")
                tf.imwrite(dst_nuclei, mask)
                print("Saving mask of %s to %s" % (external_name, dst_nuclei))
            except FileExistsError:
                print("Skipping mask of %s to %s" % (external_name, dst_nuclei))
                continue

    print("FINISHED")


if __name__ == "__main__":
    main()



from datetime import datetime
from spotipy.model import SpotNet
from cenfind.training.train import config_unet, fetch_all_fields
from cenfind.publication.constants import PREFIX_REMOTE, datasets
from cenfind.core.data import Dataset


def main():
    path_datasets = [PREFIX_REMOTE / ds for ds in datasets]
    dss = [Dataset(path) for path in path_datasets]

    all_train_x, all_train_y, all_test_x, all_test_y = fetch_all_fields(dss)

    time_stamp = datetime.now().strftime("%Y%m%d_%H%M%S")

    model_unet = SpotNet(config_unet, name=time_stamp, basedir='models/dev/unet')
    model_unet.train(all_train_x, all_train_y, validation_data=(all_test_x, all_test_y), epochs=100)

    return 0


if __name__ == '__main__':
    main()



import logging

import numpy as np
import pandas as pd
from spotipy.utils import points_matching
from stardist.models import StarDist2D
from tqdm import tqdm

from cenfind.core.data import Dataset
from cenfind.core.detectors import extract_nuclei
from cenfind.publication.constants import datasets, PREFIX_REMOTE

logging.basicConfig(level=logging.INFO, format="[%(levelname)s] %(asctime)s: %(message)s")


def main():
    model_stardist = StarDist2D.from_pretrained('2D_versatile_fluo')
    accuracies = []
    preds = []
    actual = []
    for dataset in datasets:
        path_dataset = PREFIX_REMOTE / dataset
        dataset = Dataset(path_dataset)
        for field in tqdm(dataset.fields):
            nuclei_mask = field.mask(0)
            nuclei_preds = extract_nuclei(field=field, model=model_stardist, channel=0)
            nuclei_actual = extract_nuclei(field=field, annotation=nuclei_mask, channel=0)
            centres_preds = [c.centre for c in nuclei_preds]
            centres_actual = [c.centre for c in nuclei_actual]
            logging.info("Found %d nuclei instead of %d" % (len(centres_preds), len(centres_actual)))
            preds = preds + [p.position for p in centres_preds]
            actual = actual + [p.position for p in centres_actual]
        res = points_matching(preds, actual, cutoff_distance=50)
        accuracies.append({'DATASET': dataset.path.name,
                           'F1': np.round(res.f1, 3)})
    acc_df = pd.DataFrame(accuracies)
    acc_df.to_csv('out/_nuclei_acc.csv')


if __name__ == '__main__':
    main()



import contextlib
import functools
import logging
import os
from pathlib import Path
from typing import List

import cv2
import numpy as np
import tensorflow as tf
from csbdeep.utils import normalize
from skimage import measure
from skimage.exposure import rescale_intensity
from skimage.feature import hessian_matrix, hessian_matrix_eigvals
from skimage.filters.thresholding import threshold_otsu
from spotipy.model import SpotNet
from spotipy.utils import normalize_fast2d
from stardist.models import StarDist2D

from cenfind.core.data import Field
from cenfind.core.log import get_logger
from cenfind.core.structures import Centriole, Nucleus
from cenfind.core.visualisation import draw_foci, resize_image

np.random.seed(1)
tf.random.set_seed(2)
tf.get_logger().setLevel(logging.ERROR)

os.environ['TF_CPP_MIN_LOG_LEVEL'] = '3'

logger = get_logger(__name__)


def extract_foci(field: Field, channel: int, foci_model_file: Path, prob_threshold=0.5, min_distance=2, ) -> List[
    Centriole]:
    """
    Detect centrioles in Field as row, col, row major
    :param field:
    :param foci_model_file:
    :param channel:
    :param prob_threshold:
    :param min_distance:
    :return:
    """
    logger.info("Processing %s / %d" % (field.name, channel))
    data = field.data[channel, ...]

    @functools.lru_cache(maxsize=None)
    def get_model(model):
        path = Path(model)
        if not path.is_dir():
            raise (FileNotFoundError(f"{path} is not a directory"))

        return SpotNet(None, name=path.name, basedir=str(path.parent))

    with open(os.devnull, "w") as f, contextlib.redirect_stdout(f):
        data = normalize_fast2d(data)
        model = get_model(foci_model_file)
        _, points_preds = model.predict(
            data, prob_thresh=prob_threshold, min_distance=min_distance, verbose=False
        )

    foci = [
        Centriole(field=field, channel=channel, centre=(r, c), index=f_id, label='Centriole') for f_id, (r, c) in
        enumerate(points_preds.tolist())
    ]

    centrosomes_mask = np.zeros(data.shape, dtype="uint8")
    centrosomes_mask = draw_foci(centrosomes_mask, foci, radius=min_distance * 2)

    centrosomes_map = measure.label(centrosomes_mask)
    centrosomes_centroids = measure.regionprops(centrosomes_map)

    for f in foci:
        foci_index = centrosomes_map[f.centre]
        centrosome_centroid = centrosomes_centroids[foci_index - 1].centroid
        f.parent = Centriole(field=field, channel=channel, centre=centrosome_centroid, label="Centrosome")

    if len(foci) == 0:
        logger.warning("No centrioles (channel: %s) has been detected in %s" % (channel, field.name))

    logger.info("(%s), channel %s: foci: %s" % (field.name, channel, len(foci)))
    return foci


def extract_nuclei(field: Field, channel: int, model: StarDist2D = None) -> List[Nucleus]:
    """
    Extract the nuclei from the field.
    :param field:
    :param channel:
    :param model:

    :return: List of Contours.

    """
    if model is None:
        from stardist.models import StarDist2D
        with open(os.devnull, "w") as f, contextlib.redirect_stdout(f):
            model = StarDist2D.from_pretrained("2D_versatile_fluo")

    data = field.data[channel, ...]

    data_resized = resize_image(data)
    with open(os.devnull, "w") as f, contextlib.redirect_stdout(f):
        labels, _ = model.predict_instances(normalize(data_resized))
    labels = cv2.resize(
        labels, dsize=data.shape, fx=1, fy=1, interpolation=cv2.INTER_NEAREST
    )

    if len(labels) == 0:
        logger.warning("No nucleus has been detected in %s" % field.name)
        return []
    labels_id = np.unique(labels)

    nuclei = []
    for nucleus_index, nucleus_label in enumerate(labels_id):
        if nucleus_label == 0:
            continue
        sub_mask = np.zeros_like(labels, dtype="uint8")
        sub_mask[labels == nucleus_label] = 1
        contour, _ = cv2.findContours(
            sub_mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE
        )
        nucleus = Nucleus(field=field, channel=channel, contour=contour[0], label="Nucleus", index=nucleus_index - 1)
        nuclei.append(nucleus)

    logger.info("(%s), channel %s: foci: %s" % (field.name, channel, len(nuclei)))

    return nuclei


def extract_cilia(field: Field, channel, sigma=5.0, eccentricity=.9, area=200) -> List[Centriole]:
    data = field.data[channel, ...]
    resc = rescale_intensity(data, out_range='uint8')

    h_elems = hessian_matrix(resc, sigma=sigma, order='rc')
    _, minima_ridges = hessian_matrix_eigvals(h_elems)
    threshold = threshold_otsu(minima_ridges)

    mask = minima_ridges < threshold
    labels = measure.label(mask)
    props = measure.regionprops(labels, mask)

    if not props:
        logger.warning("No cilium (channel: %s) has been detected in %s" % (channel, field.name))

    result = []
    for prop in props:
        if prop.eccentricity > eccentricity and prop.area > area:
            r, c = prop.centroid
            result.append(Centriole(field=field, channel=channel, centre=(int(r), int(c)), label='Cilium'))

    return result



import logging
import uuid

from labelbox import (
    OntologyBuilder,
    LabelingFrontend,
    Dataset,
    Project,
    MALPredictionImport
)
from labelbox.data.annotation_types import (
    ObjectAnnotation,
    Point,
    LabelList,
    Label,
    ImageData,
)
from labelbox.data.serialization import NDJsonConverter

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


def to_labelbox_format(predictions):
    """
    Convert numpy coordinates into Labelbox format.
    :param predictions:
    :return:
    """
    annotations = []
    for r, c in predictions:
        annot = ObjectAnnotation(name='Point', value=Point(x=c, y=r))
        annotations.append(annot)
    return annotations


def label_create(image, predictions, external_id):
    """
    Combine an image and its annotation into a Label.
    :param predictions:
    :param image:
    :return:
    """
    label = Label(data=ImageData(arr=image),
                  annotations=to_labelbox_format(predictions))
    label.data.external_id = external_id
    return label


def labels_list_create(labels):
    """
    Combine all labels into a LabelList object.
    :return:
    """
    labels_list = LabelList()
    for label in labels:
        labels_list.append(label)
    return labels_list


def task_prepare(client, project, dataset, labels_list):
    signer = lambda _bytes: client.upload_data(content=_bytes, sign=True)

    # labels_list.assign_feature_schema_ids(OntologyBuilder.from_project(project))
    labels_list.add_to_dataset(dataset, signer)

    mal_ndjson = list(NDJsonConverter.serialize(labels_list))
    upload_job = MALPredictionImport.create_from_objects(
        client=client,
        project_id=project.uid,
        name=f'upload-job-{uuid.uuid4()}',
        predictions=mal_ndjson)

    return upload_job


def project_create(client, project_name):
    """
    Create a project object or return any existing with `project name`.
    :param client:
    :param project_name:
    :return:
    """
    projects_test = client.get_projects(where=Project.name == project_name)

    try:
        project = next(projects_test)
        logger.debug('Project test found (%s)', project.uid)
        return project
    except StopIteration:
        logger.debug('No such project; creating...')
    finally:
        project = client.create_project(name=project_name, description='')
        return project


def ontology_setup(client, project, ontology_id):
    """
    Fetch the ontology using the ID and attach it to the project.
    This is in place.
    :param client:
    :param project:
    :param ontology_id:
    :return:
    """
    ontology = client.get_ontology(ontology_id=ontology_id)
    ontology_builder = OntologyBuilder.from_ontology(ontology)
    editor = next(
        client.get_labeling_frontends(where=LabelingFrontend.name == 'editor'))
    project.setup(editor, ontology_builder.asdict())


def dataset_create(client, dataset_name):
    """
    Create a dataset object and delete any dataset with same name.
    :param client:
    :param dataset_name:
    :return:
    """
    datasets_test = client.get_datasets(where=Dataset.name == dataset_name)
    try:
        dataset = next(datasets_test)
        logger.debug('Found a test dataset (%s)', dataset.name)
        dataset.delete()
    except StopIteration:
        logger.debug('No such test dataset; creating one...')
    finally:
        dataset = client.create_dataset(name=dataset_name, iam_integration=None)
    return dataset



from collections import defaultdict
from pathlib import Path

import labelbox
from dotenv import dotenv_values
from tqdm import tqdm

config = dotenv_values('.env')

dst = Path('/data1/centrioles/')
dst.mkdir(exist_ok=True)

def main():
    lb = labelbox.Client(api_key=config['LABELBOX_API_KEY'])
    project = lb.get_project(config['PROJECT_CENTRIOLES'])
    labels = project.label_generator()

    container = defaultdict(list)

    for label in tqdm(labels):
        ds = label.extra['Dataset Name']
        external_name = label.data.external_id
        # foci_in_label = [lab for lab in label.annotations if lab.name == 'Point']
        stem = external_name.split('.')[0]
        channel = stem.split('_')[-1]
        channel_id = channel[-1]
        field_name = '_'.join(stem.split('_')[:-2])
        container[ds].append(f"{field_name},{channel_id}")

    for ds in container.keys():
        with open(dst / ds / f'_fields_channel.txt', 'w') as f:
            for line in container[ds]:
                f.write(line + "\n")
        with open(dst / ds / f'_fields.txt', 'w') as f:
            for line in container[ds]:
                field, channel = line.split(',')
                f.write(field + "\n")


if __name__ == '__main__':
    main()



from typing import Tuple

import cv2
import numpy as np
from attrs import define

from cenfind.core.data import Field
from cenfind.core.log import get_logger

logger = get_logger(__name__)


@define
class Centriole:
    """
    Represent a centriole as a 2d point associated with a Field and a channel index.
    """
    field: Field
    channel: int
    centre: Tuple[int, int]
    index: int = 0
    label: str = ""
    parent: 'Centriole' = None

    @property
    def centre_xy(self) -> Tuple[int, int]:
        y, x = self.centre
        return x, y

    def intensity(self, image: np.ndarray, k: int = 0, channel: int = None) -> int:
        r, c = self.centre
        max_r, max_c = image.shape[-2:]

        r_start = np.clip(r - k, 0, max_r)
        r_stop = np.clip(r + k, 0, max_r)
        c_start = np.clip(c - k, 0, max_c)
        c_stop = np.clip(c + k, 0, max_c)

        if image.ndim < 3:
            return int(np.sum(image[r_start:r_stop, c_start:c_stop]))
        if channel is None:
            raise ValueError('Channel must be supplied when image has 3 dimensions')
        return int(np.sum(image[channel, r_start:r_stop, c_start:c_stop]))

    def as_dict(self) -> dict:
        return {
            "channel": self.channel,
            "pos_r": self.centre[0],
            "pos_c": self.centre[1],
            "intensity": self.intensity(self.field.data[self.channel, ...], k=1)
        }


@define
class Nucleus:
    field: Field
    channel: int
    contour: np.ndarray
    index: int = 0
    label: str = ""

    @property
    def centre(self) -> Tuple[int, int]:
        moments = cv2.moments(self.contour)
        centre_x = int(moments["m10"] / (moments["m00"] + 1e-5))
        centre_y = int(moments["m01"] / (moments["m00"] + 1e-5))
        return int(centre_y), int(centre_x)

    @property
    def centre_xy(self) -> Tuple[int, int]:
        y, x = self.centre
        return int(x), int(y)

    @property
    def intensity(self) -> int:
        _data = self.field.data[self.channel, ...]
        mask = np.zeros_like(_data)
        cv2.drawContours(mask, [self.contour], 0, 255, -1)
        masked = cv2.bitwise_and(_data, mask)
        return int(np.sum(masked))

    @property
    def area(self) -> int:
        return int(cv2.contourArea(self.contour))

    @property
    def full_in_field(self) -> bool:
        """
        Check if a contour is fully visible.
        Parameters
        ----------
        nucleus
        image_shape
        fraction

        Returns
        -------

        """
        h, w = self.field.data.shape[-2:]
        fraction = 0.05
        pad_lower = int(fraction * h)
        pad_upper = h - pad_lower
        centroid = self.centre
        if all([pad_lower < c < pad_upper for c in centroid]):
            return True
        return False

    def as_dict(self) -> dict:
        result = {
            "channel": self.channel,
            "pos_r": self.centre[0],
            "pos_c": self.centre[1],
            "intensity": self.intensity,
            "surface_area": self.area,
            "is_nucleus_full": self.full_in_field,
            "contour": self.contour.tolist()
        }
        return result



import argparse
import os
from pathlib import Path

import pandas as pd
from tqdm import tqdm

from cenfind.core.data import Dataset
from cenfind.core.detectors import extract_foci, extract_nuclei, extract_cilia
from cenfind.core.measure import Assigner
from cenfind.core.serialise import (
    save_points,
    save_contours,
    save_assigned,
    save_assigned_centrioles,
    save_visualisation
)
from cenfind.core.statistics import proportion_cilia, frequency
from cenfind.core.visualisation import visualisation, create_vignette


def register_parser(parent_subparsers):
    parser = parent_subparsers.add_parser(
        "score", help="Score the projections given the channels"
    )
    parser.add_argument("dataset", type=Path, help="Path to the dataset")
    parser.add_argument("model", type=Path, help="Absolute path to the model folder")
    parser.add_argument(
        "--channel_nuclei",
        "-n",
        type=int,
        required=True,
        help="Channel index for nuclei segmentation, e.g., 0 or 3",
    )
    parser.add_argument(
        "--channel_centrioles",
        "-c",
        nargs="+",
        type=int,
        default=[],
        help="Channel indices to analyse, e.g., 1 2 3",
    )
    parser.add_argument(
        "--channel_cilia",
        "-l",
        type=int,
        help="Channel indices to analyse cilium",
    )
    parser.add_argument(
        "--vicinity",
        "-v",
        type=int,
        default=50,
        help="Distance threshold in pixel (default: 50 px)",
    )

    parser.add_argument("--cpu", action="store_true", help="Only use the cpu")

    return parser


def run(args):
    if (args.channel_centrioles is None) and (args.channel_cilia is None):
        raise ValueError("Please specify at least one channel to evaluate.")
    if args.cpu:
        os.environ["CUDA_VISIBLE_DEVICES"] = "-1"

    dataset = Dataset(args.dataset)
    dataset.setup()

    pbar = tqdm(dataset.fields)

    ciliated_container = []
    results = {}
    for field in pbar:
        pbar.set_description(f"{field.name}")

        nuclei = extract_nuclei(field, args.channel_nuclei)
        save_contours(dataset.nuclei / f"{field.name}_C{args.channel_nuclei}.json", nuclei)

        pbar_dict = {"nuclei": len(nuclei)}
        if args.channel_centrioles is not None:
            for channel in args.channel_centrioles:
                pbar_dict["channel"] = channel

                centrioles = extract_foci(field=field, channel=channel, foci_model_file=args.model)

                assignment = Assigner(centrioles, nuclei, vicinity=args.vicinity)
                centrioles_nuclei = assignment.assign_centrioles()
                scores = assignment.score_nuclei(field.name, channel)

                background = create_vignette(field, marker_index=channel, nuclei_index=args.channel_nuclei)
                vis = visualisation(background, centrioles=centrioles, nuclei=nuclei, assigned=assignment.assignment)

                results[(field.name, channel)] = {
                    'scores': scores,
                    'assignment': assignment.assignment,
                    'centrioles_nuclei': centrioles_nuclei,
                    'centrioles': centrioles,
                    'visualisation': vis}

                pbar_dict[f"centrioles"] = len(centrioles)
                pbar.set_postfix(pbar_dict)

        if args.channel_cilia is not None:
            channel = args.channel_cilia
            ciliae = extract_cilia(field, channel=channel)
            record = proportion_cilia(field, ciliae, nuclei, channel)
            ciliated_container.append(record)

            save_points(dataset.cilia / f"{field.name}_C{channel}.tsv",
                      ciliae)

            pbar_dict["cilia"] = len(ciliae)
            pbar.set_postfix(pbar_dict)

    for (field, channel), data in results.items():
        save_points(dataset.centrioles / f"{field}_C{channel}.tsv", data['centrioles'])
        save_assigned(dataset.assignment / f"{field}_C{channel}_matrix.txt", data['assignment'])
        save_assigned_centrioles(dataset.statistics / f"{field}_C{channel}_assigned.tsv", data['centrioles_nuclei'])
        save_visualisation(dataset.visualisation / f"{field}_C{channel}.png", data['visualisation'])

    scores_all = pd.concat([v['scores'] for v in results.values()])
    binned = frequency(scores_all)
    binned.to_csv(dataset.statistics / "statistics.tsv", sep="\t", index=True)

    if ciliated_container:
        ciliated_all = pd.concat(ciliated_container)
        ciliated_all.to_csv(dataset.statistics / "ciliated.tsv", sep="\t", index=True)


if __name__ == "__main__":
    args = argparse.Namespace(dataset=Path('data/dataset_test'),
                              model=Path('models/master'),
                              channel_nuclei=0,
                              channel_centrioles=[1, 2],
                              channel_cilia=3,
                              vicinity=50,
                              cpu=False,
                              )
    run(args)



import json
from datetime import datetime
from pathlib import Path

import numpy as np
import tensorflow as tf
from numpy.random import seed
from spotipy.model import SpotNet, Config
from tqdm import tqdm

from cenfind.training.config import config_multiscale, transforms
from cenfind.core.data import Dataset

seed(1)
tf.random.set_seed(2)


def read_config(path):
    """
    Read config json file.
    :param path:
    :return:
    """
    with open(path, "r") as config:
        config_dict = json.load(config)
    return Config(**config_dict)


def fetch_all_fields(datasets: list[Dataset]):
    all_train_x = []
    all_train_y = []

    all_test_x = []
    all_test_y = []

    for ds in tqdm(datasets):
        train_x, train_y = ds.load_pairs(ds, split="train", transform=transforms)
        test_x, test_y = ds.load_pairs(ds, split="test")
        all_train_x.append(train_x)
        all_train_y.append(train_y)
        all_test_x.append(test_x)
        all_test_y.append(test_y)

    all_train_x = np.concatenate(all_train_x, axis=0)
    all_train_y = np.concatenate(all_train_y, axis=0)

    all_test_x = np.concatenate(all_test_x, axis=0)
    all_test_y = np.concatenate(all_test_y, axis=0)

    return all_train_x, all_train_y, all_test_x, all_test_y


def register_parser(parent_subparsers):
    parser = parent_subparsers.add_parser(
        "train", help="Train a Spotnet model using the datasets"
    )
    parser.add_argument("datasets", type=Path, nargs="+", help="Path to the dataset")
    parser.add_argument(
        "--model_path", type=Path, required=True, help="Path to the model fit"
    )
    parser.add_argument("--epochs", type=int, required=True, help="Number of epochs")

    return parser


def run(args):
    datasets = [Dataset(path) for path in args.datasets]

    all_train_x, all_train_y, all_test_x, all_test_y = fetch_all_fields(datasets)

    time_stamp = datetime.now().strftime("%Y%m%d_%H%M%S")
    model_multiscale = SpotNet(
        config_multiscale, name=time_stamp, basedir=args.model_path
    )

    model_multiscale.train(
        all_train_x,
        all_train_y,
        validation_data=(all_test_x, all_test_y),
        epochs=args.epochs,
    )



import cv2
import pandas as pd

from cenfind.core.data import Dataset
from cenfind.core.visualisation import draw_foci
from cenfind.publication.constants import datasets, PREFIX_REMOTE
from cenfind.publication.detectors_other import log_skimage, simpleblob_cv2
from cenfind.core.detectors import extract_foci


def main():
    methods = [
        ['unet', extract_foci, 'models/dev/unet/20221116_193624'],
        ['multiscale', extract_foci, 'models/dev/multiscale/20221116_160118'],
        ['log_skimage', log_skimage, None],
        ['simpleblob_cv2', simpleblob_cv2, None],
    ]
    perfs = []
    for ds_name in datasets:
        ds = Dataset(PREFIX_REMOTE / ds_name)
        for field, channel in ds.pairs(split='test'):
            vis = field.channel(channel)
            annotation = field.annotation(channel)

            for name, method, model_path in methods:
                foci, f1 = field.run_detection(method, annotation=annotation, channel=channel,
                                         model_path=model_path, tolerance=3)
                print(f"{field.name} using {name}: F1={f1} (foci detected: {len(foci)})")

                perf = {'dataset': field.dataset.path.name,
                        'field': field.name,
                        'channel': channel,
                        'method': name,
                        'f1': round(f1, 3)}
                perfs.append(perf)

                mask = draw_foci(vis, foci)
                cv2.imwrite(f'out/images/{field.name}_max_C{channel}_preds_{name}.png', mask)

    perfs_df = pd.DataFrame(perfs)
    perfs_df.to_csv('out/perfs_blobdetectors.csv')

    summary = perfs_df.groupby('method')['f1'].agg(['mean', 'std']).round(3)
    summary = summary.reset_index()
    summary['F1 (mean ± st. dev.)'] = summary['mean'].astype(str) + '±' + summary['std'].astype(str)
    summary.columns = ['Method', 'F1', 'Standard deviation', 'F1 (mean ± st. dev.)']
    summary.to_csv('out/perfs_blobdetectors_summary.csv')


if __name__ == '__main__':
    main()



from pathlib import Path
import argparse
from dotenv import dotenv_values
from labelbox import Client
import sys

from cenfind.core.data import Dataset


def register_parser(parent_subparsers):
    parser = parent_subparsers.add_parser(
        "upload", help="Upload the vignettes to labelbox"
    )
    parser.add_argument(
        "dataset", type=Path, help="Path to the dataset with existing vignettes"
    )
    parser.add_argument("--env", type=Path, help="Path to the .env file")

    return parser


def run(args):
    ds = Dataset(args.dataset)

    config = dotenv_values(args.env)
    client = Client(api_key=config["LABELBOX_API_KEY"])

    dataset = client.create_dataset(name=f"{ds.path.name}", iam_integration=None)

    asset = [
        {"row_data": str(path), "external_id": path.name}
        for path in sorted((ds.path / "vignettes").iterdir())
    ]
    if not asset:
        print('There are no vignettes...')
        sys.exit()
    dataset.create_data_rows(asset)


if __name__ == "__main__":
    args = argparse.Namespace(dataset=Path('data/dataset_test'),
                              env=Path('.env'))
    run(args)



import argparse
import logging
from pathlib import Path
from tqdm import tqdm
import cv2
from dotenv import dotenv_values
from labelbox import Client, MediaType

from labelbox.schema.queue_mode import QueueMode
from labelbox.exceptions import ResourceNotFoundError

from cenfind.core.data import Dataset
from cenfind.core.detectors import extract_foci
from cenfind.training.helpers import (
    ontology_setup,
    label_create,
    labels_list_create,
    task_prepare,
)

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
foci_model = Path("models/master/")

path_dotenv = Path(".env")
if not path_dotenv.exists():
    FileNotFoundError(path_dotenv.resolve())

config = dotenv_values(path_dotenv)


def register_parser(parent_subparsers):
    parser = parent_subparsers.add_parser(
        "uploadmal",
        help="Upload MAL",
    )
    parser.add_argument("dataset", type=Path, help="Path to the dataset folder")
    parser.add_argument("model", type=Path, help="Path to the model")

    return parser


def run(args):
    client = Client(api_key=config["LABELBOX_API_KEY"])

    project_name = "centrioles_mal"
    project_id = config["PROJECT_CENTRIOLES_MAL"]
    ontology_id = "cl6gk46xv4wjc07yt4kyygxh1"

    try:
        project = client.get_project(project_id)
    except ResourceNotFoundError:
        print("Project not existing, creating one...")
        project = client.create_project(
            name=project_name,
            queue_mode=QueueMode.Dataset,
            # Quality Settings setup
            auto_audit_percentage=1,
            auto_audit_number_of_labels=1,
            media_type=MediaType.Image,
        )
        ontology_setup(client, project, ontology_id=ontology_id)
    project.enable_model_assisted_labeling()

    ds = Dataset(args.dataset)
    dataset_name = ds.path.name

    dataset = client.create_dataset(name=dataset_name, iam_integration=None)
    project.datasets.connect(dataset)

    labels = []
    for field in tqdm(ds.fields):
        for channel_id in range(1, 4):
            mal_label = extract_foci(field, channel_id, foci_model)
            vignette_path = (
                    ds.path / "vignettes" / f"{field.name}_max_C{channel_id}.png"
            )
            image = cv2.imread(str(vignette_path))
            image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
            labels.append(label_create(image, mal_label, vignette_path.name))
    mal_label_list = labels_list_create(labels)
    task = task_prepare(client, project, dataset, mal_label_list)
    print(task.errors)


if __name__ == "__main__":
    args = argparse.Namespace(dataset=Path('data/dataset_test'),
                              )
    run(args)



from numpy.random import seed

seed(1)

import tensorflow as tf

tf.random.set_seed(2)
from datetime import datetime
from spotipy.model import SpotNet
from spotipy.model import Config
from cenfind.publication.constants import PREFIX_REMOTE
from cenfind.core.data import Dataset
from cenfind.training.train import load_pairs, transforms

config = Config(
    n_channel_in=1,
    backbone="unet",
    mode="mae",
    unet_n_depth=3,
    unet_pool=4,
    unet_n_filter_base=64,
    spot_weight=40,
    multiscale=True,
    train_learning_rate=3e-4,
    train_foreground_prob=1,
    train_batch_norm=False,
    train_multiscale_loss_decay_exponent=1,
    train_patch_size=(512, 512),
    spot_weight_decay=0.5,
    train_batch_size=2,
)


def main():
    ds = Dataset(PREFIX_REMOTE / 'centrioles', projection_suffix='_max')

    train_x, train_y = load_pairs(ds, split="train", transform=transforms, sigma=1.5)
    validation_x, validation_y = load_pairs(ds, split="validation", sigma=5)
    test_x, test_y = load_pairs(ds, split="test", sigma=1.5)

    time_stamp = datetime.now().strftime("%Y%m%d_%H%M%S")

    model = SpotNet(config, name=f"multi_depth3_bce_sigma5_{time_stamp}", basedir='models/dev/multiscale')
    model.train(train_x, train_y, validation_data=(test_x, test_y), epochs=50)

    return 0


if __name__ == '__main__':
    main()



import pandas as pd
from matplotlib import pyplot as plt

from cenfind.publication.constants import ROOT_DIR

pd.options.display.max_columns = 50


def precision(x):
    return x['tp'] / (x['tp'] + x['fp'])


def recall(x):
    return x['tp'] / (x['tp'] + x['fn'])


def main():
    run = '20221116_160118'
    path_data = ROOT_DIR / f'out/perfs_{run}.csv'
    data = pd.read_csv(path_data)
    data = data.drop(['precision', 'recall', 'f1'], axis=1)
    data_tol_3 = data.loc[data['tolerance'] == 3]
    grouped = data_tol_3.groupby('threshold')[['tp', 'fp', 'fn']].sum()
    grouped['precision'] = grouped.apply(precision, axis=1)
    grouped['recall'] = grouped.apply(recall, axis=1)
    precision0 = pd.DataFrame([[0, 0, 0, 1, 0]],
                              columns=['tp', 'fp', 'fn', 'precision', 'recall'], index=[1])
    grouped = pd.concat([grouped, precision0]).sort_values('recall')
    grouped = grouped.reset_index()
    grouped = grouped.rename({'index': 'threshold'}, axis=1)
    grouped.to_csv(ROOT_DIR / f'out/pr_data_{run}.csv')

    fig, ax = plt.subplots(1, 1, figsize=(5, 5))
    ax.scatter(grouped['recall'], grouped['precision'], marker='.', color='blue',
               alpha=grouped['threshold'].to_numpy())
    ax.spines.right.set_visible(False)
    ax.spines.top.set_visible(False)
    ax.set_xlim(0, 1)
    ax.set_ylim(0, 1)
    ax.set_xlabel('recall')
    ax.set_ylabel('precision')
    ax.set_title("PR Curve\nall DS; any channel")
    fig.savefig(ROOT_DIR / f'out/checks/precision_recall_all_{run}.png', dpi=300)


if __name__ == '__main__':
    main()



import cv2
import pandas as pd
from dotenv import dotenv_values
from tqdm import tqdm

from cenfind.core.data import Dataset
from cenfind.core.detectors import extract_nuclei
from cenfind.core.measure import flag, full_in_field
from cenfind.core.visualisation import draw_contour, create_vignette
from cenfind.publication.constants import datasets, PREFIX_REMOTE

config = dotenv_values('.env')

pd.set_option('display.max_rows', None)
pd.set_option('display.max_columns', None)
pd.set_option('display.width', None)
pd.set_option('display.max_colwidth', None)


def main():
    records = []
    for dataset in datasets:
        dataset = Dataset(PREFIX_REMOTE / dataset)
        for field in tqdm(dataset.fields):
            mask = field.mask(0)
            nuclei = extract_nuclei(field, 0)
            vignette = create_vignette(field, 1, 0)
            for nucleus in nuclei:
                centre = nucleus.centre
                is_full = full_in_field(centre.centre, mask, .05)
                records.append({'dataset': dataset.path.name,
                                'field': field.name,
                                'centre': centre.centre,
                                'is_full': is_full})
                draw_contour(vignette, nucleus, color=flag(is_full))

            cv2.imwrite(f'out/checks/{field.name}.png', vignette)
    df = pd.DataFrame(records)

    summary = df.groupby(['dataset'])['is_full'].agg(['count', sum])
    summary = summary.reset_index()
    summary.columns = ['Dataset', 'Nuclei detected', 'Nuclei full']
    summary.to_csv('out/nuclei_counts.csv')


if __name__ == '__main__':
    main()



from typing import List, Tuple

import cv2
import numpy as np
from skimage.draw import disk
from skimage.exposure import rescale_intensity

from cenfind.core.data import Field
from cenfind.core.log import get_logger
from cenfind.core.structures import Centriole, Nucleus

logger = get_logger(__name__)


def resize_image(image: np.ndarray, factor: int = 256) -> np.ndarray:
    """
    Resize the image for nuclei segmentation by StarDist
    :param image: a single channel image (H x W)
    :param factor: the target dimension
    :return the image resized
    """
    height, width = image.shape
    shrinkage_factor = int(height // factor)
    height_scaled = int(height // shrinkage_factor)
    width_scaled = int(width // shrinkage_factor)
    data_resized = cv2.resize(
        image,
        dsize=(height_scaled, width_scaled),
        fx=1,
        fy=1,
        interpolation=cv2.INTER_NEAREST,
    )
    return data_resized


def draw_point(image: np.ndarray, point: Centriole,
               color=(0, 255, 0),
               annotation=True,
               marker_type=cv2.MARKER_SQUARE,
               marker_size=8,
               ):
    r, c = point.centre
    offset_col = int(0.01 * image.shape[1])

    if annotation:
        cv2.putText(
            image,
            f"{point.label} {point.index}",
            org=(c + offset_col, r),
            fontFace=cv2.FONT_HERSHEY_SIMPLEX,
            fontScale=0.4,
            thickness=1,
            color=color,
        )

    return cv2.drawMarker(
        image, (c, r), color, markerType=marker_type, markerSize=marker_size
    )


def draw_foci(image: np.ndarray, foci: list[Centriole], radius=2) -> np.ndarray:
    """
    Draw foci as disks of given radius
    :param image: the channel for the dimension extraction
    :param foci: the list of foci
    :param radius: the radius of foci
    :return the mask with foci
    """
    mask = np.zeros(image.shape, dtype="uint8")
    for f in foci:
        rr, cc = disk(f.centre, radius, shape=image.shape[-2:])
        mask[rr, cc] = 250
    return mask


def draw_contour(image: np.ndarray, nucleus: Nucleus, color=(0, 255, 0), annotation=True, thickness=2):
    r, c = nucleus.centre
    cv2.drawContours(image, [nucleus.contour], -1, color, thickness=thickness)
    if annotation:
        cv2.putText(
            image,
            f"{nucleus.label}{nucleus.index}",
            org=(c, r),
            fontFace=cv2.FONT_HERSHEY_SIMPLEX,
            fontScale=0.8,
            thickness=2,
            color=color,
        )
        cv2.drawMarker(
            image, (c, r), color, markerType=cv2.MARKER_STAR, markerSize=10
        )
    return image


def _color_channel(data: np.ndarray, color: tuple, out_range: str):
    """
    Create a colored version of a channel image
    :param data: the data to use
    :param color: the color as a tuple (B, G, R)
    :param out_range:
    :return:
    """
    data = rescale_intensity(data, out_range=out_range)
    b = np.multiply(data, color[0], casting="unsafe")
    g = np.multiply(data, color[1], casting="unsafe")
    r = np.multiply(data, color[2], casting="unsafe")
    res = cv2.merge([b, g, r])
    return res


def create_vignette(field: Field, marker_index: int, nuclei_index: int):
    """
    Normalise all markers
    Represent them as blue
    Highlight the channel in green
    :param field:
    :param nuclei_index:
    :param marker_index:
    :return:
    """
    layer_nuclei = field.data[nuclei_index, ...]
    layer_marker = field.data[marker_index, ...]

    nuclei = _color_channel(layer_nuclei, (1, 0, 0), "uint8")
    marker = _color_channel(layer_marker, (0, 1, 0), "uint8")

    res = cv2.addWeighted(marker, 1, nuclei, 0.5, 0)
    res = cv2.putText(
        res,
        f"{field.name} {marker_index}",
        (100, 100),
        cv2.FONT_HERSHEY_SIMPLEX,
        0.8,
        (255, 255, 255),
        1,
        cv2.LINE_AA,
    )
    return res


def visualisation(background: np.ndarray,
                  centrioles: List[Centriole],
                  nuclei: List[Nucleus],
                  assigned: List[Tuple[Centriole, Nucleus]] = None,
                  ) -> np.ndarray:
    for centriole in centrioles:
        background = draw_point(background, centriole, annotation=False)
    for nucleus in nuclei:
        background = draw_contour(background, nucleus, annotation=False)
    if assigned is None:
        return background

    it = np.nditer(assigned, flags=['multi_index'])
    for entry in it:
        if not entry:
            continue
        n, c = it.multi_index
        nucleus = nuclei[n]
        centriole = centrioles[c]
        background = draw_contour(background, nucleus, annotation=False)
        background = draw_point(background, centriole, annotation=False)
        cv2.arrowedLine(background, tuple(reversed(centriole.centre)), tuple(reversed(nucleus.centre)),
                        color=(0, 255, 0), thickness=2)

    return background



from typing import Tuple
import cv2
import numpy as np
from skimage.exposure import rescale_intensity
from skimage.feature import blob_log
from spotipy.utils import points_matching

from cenfind.core.data import Field
from cenfind.core.structures import Point


def blob2point(keypoint: cv2.KeyPoint) -> tuple[int, ...]:
    res = (int(keypoint.pt[1]), int(keypoint.pt[0]))
    return res


def log_skimage(data: Field, channel: int, **kwargs) -> list:
    data = data.channel(channel)
    data = rescale_intensity(data, out_range=(0, 1))
    foci = blob_log(data, min_sigma=1, max_sigma=2, num_sigma=10, threshold=0.1)
    res = [(int(r), int(c)) for r, c, _ in foci]

    return res


def simpleblob_cv2(data: Field, channel: int, **kwargs) -> list:
    data = data.channel(channel)
    foci = rescale_intensity(data, out_range="uint8")
    params = cv2.SimpleBlobDetector_Params()

    params.blobColor = 255
    params.filterByArea = True
    params.minArea = 5
    params.maxArea = 100
    params.minDistBetweenBlobs = 1
    params.minThreshold = 0
    params.maxThreshold = 255

    detector = cv2.SimpleBlobDetector_create(params)
    keypoints = detector.detect(foci)

    res = [blob2point(kp) for kp in keypoints]

    return res


def run_detection(
    method,
    data: Field,
    annotation: np.ndarray,
    tolerance,
    channel=None,
    model_path=None,
) -> Tuple[list[Point], float]:
    _foci = method(data, foci_model_file=model_path, channel=channel)
    res = points_matching(annotation, _foci, cutoff_distance=tolerance)
    f1 = np.round(res.f1, 3)
    foci = [Point((r, c), label="Point") for r, c in _foci]
    return foci, f1



import re
import matplotlib
import pandas as pd
from matplotlib import pyplot as plt

from cenfind.publication.constants import datasets, pattern_dataset, protein_names, celltype_names

font = {'size': 6}
matplotlib.rc('font', **font)

matplotlib.use("Agg")

def extract_info(pattern: re, dataset_name: str):
    res = re.match(pattern, dataset_name)
    res_dict = res.groupdict()
    markers = res_dict['markers'].split('+')
    res_dict['markers'] = tuple(markers)

    return res_dict

def _setup_ax(ax):
    ax.set_ylim(0, 1)
    ax.spines['right'].set_visible(False)

    return ax


def plot_one_dataset(ax, data, metadata, title, accuracy_thresholds=(0, .5, .75, .9, 1.)):
    """
    Plot the accuracy on the specified ax
    :param metadata:
    :param ax:
    :param data:
    :param title:
    :param accuracy_thresholds:
    :return:
    """
    line_types = ['-', '--', ':']
    ax = _setup_ax(ax)
    for channel in data.channel.unique():
        channel = int(channel) - 1
        protein_code = metadata["markers"][channel]
        protein_name = protein_names[protein_code]
        marker_name = f"{protein_name}"
        line_type = line_types[channel]
        sub = data.loc[data['channel'] == channel + 1]
        ax.plot(sub['tolerance'], sub['f1'],
                color='#BF3F3F', ls=line_type, lw=1, label=marker_name)
    ax.set_xlim(0, 5)
    ax.set_xlabel('[pixel]\nTolerance')
    ax.set_ylabel('Accuracy [F-1 score]')

    ax.set_yticks(accuracy_thresholds)
    ax.set_title(title)
    ax.legend(loc='lower right', frameon=False)
    return ax


def plot_many_datasets(performances: str, metadata: dict, threshold=.5):
    data = pd.read_csv(performances)
    fig, axs = plt.subplots(nrows=1, ncols=len(datasets), figsize=(2 * len(datasets), 2))
    accuracy_thresholds = (0, .5, .75, .9, 1.)

    def scaler(x):
        return x * 102.5

    for col, ds in enumerate(datasets):
        cell_type = metadata[ds]['cell_type']
        title = f"{celltype_names[cell_type]} {metadata[ds]['treatment'] or ''}"
        sub = data.loc[(data['dataset'] == ds) & (data['threshold'] == threshold)]
        ax = axs[col]
        ax2 = ax.secondary_xaxis("top", functions=(scaler, scaler))
        ax2.set_xlabel('[nm]')
        plot_one_dataset(ax, sub, metadata[ds], title, accuracy_thresholds)
    fig.tight_layout()

    return fig


def main():
    model_name = 'multi_depth3_bce_sigma3_20230215_111734'
    path_perfs = f'out/perfs_{model_name}.csv'
    metadata = {}
    for dataset_name in datasets:
        metadata[dataset_name] = extract_info(pattern_dataset, dataset_name)

    fig = plot_many_datasets(path_perfs, metadata)
    fig.savefig(f'out/accuracy_resolution_{model_name}.png', dpi=300)


if __name__ == '__main__':
    main()



import sys
import argparse
from pathlib import Path

import pandas as pd
import tifffile as tf

from cenfind.core.data import Dataset
from cenfind.core.visualisation import visualisation
from cenfind.core.log import get_logger


def register_parser(parent_subparsers):
    parser = parent_subparsers.add_parser(
        "evaluate", help="Evaluate the model on the test split of the dataset"
    )
    parser.add_argument("dataset", type=Path, help="Path to the dataset folder")
    parser.add_argument("model", type=Path, help="Path to the model")
    parser.add_argument(
        "--channel_nuclei",
        "-n",
        type=int,
        help="Channel index of nuclei",
    )
    parser.add_argument(
        "--tolerance",
        type=int,
        nargs="+",
        default=3,
        help="Distance in pixels below which two points are deemed matching",
    )
    parser.add_argument(
        "--threshold",
        type=float,
        default=.5,
        help="Confidence."
    )
    parser.add_argument(
        "--performances_file",
        type=Path,
        help="Path of the performance file, STDOUT if not specified",
    )

    return parser


def run(args):
    dataset = Dataset(args.dataset)
    dataset.setup()
    logger = get_logger(__name__, console=1, file=dataset.logs / "cenfind.log")
    if not any(dataset.path_annotations_centrioles.iterdir()):
        logger.error(
            f"The dataset {dataset.path.name} has no annotation. You can run `cenfind predict` instead"
        )
        sys.exit(2)

    if type(args.tolerance) == int:
        tolerances = [args.tolerance]
    else:
        tolerances = args.tolerance

    path_visualisation_model = dataset.visualisation / args.model.name
    path_visualisation_model.mkdir(exist_ok=True)

    from cenfind.core.measure import evaluate
    from cenfind.core.detectors import extract_foci, extract_nuclei

    perfs = []
    pairs = dataset.splits()
    for field, channel in pairs["test"]:
        annotation = field.annotation(channel)
        predictions = extract_foci(field, channel, args.model, prob_threshold=args.threshold)
        nuclei = extract_nuclei(field, args.channel_nuclei)

        for tol in tolerances:
            logger.info("Processing %s %s %s" % (field, channel, tol))
            perf = evaluate(field, channel, annotation, predictions, tol, threshold=args.threshold)
            vis = visualisation(field=field, channel_centrioles=channel, nuclei=nuclei,
                                channel_nuclei=args.channel_nuclei)
            tf.imwrite(path_visualisation_model / f"{field.name}_C{channel}_pred.png", vis)
            perfs.append(perf)

    performance_df = pd.DataFrame(perfs)
    performance_df = performance_df.set_index("field")

    if args.performances_file:
        performance_df.to_csv(args.performances_file)
        print(performance_df)
        logger.info("Performances have been saved under %s" % args.performances_file)
    else:
        print(performance_df)
        logger.warning("Performances are ONLY displayed, not saved")


if __name__ == "__main__":
    args = argparse.Namespace(dataset=Path('data/dataset_test'),
                              model=Path('models/master'),
                              channel_nuclei=0,
                              tolerance=3,
                              threshold=.5,
                              performances_file="out/perfs.txt"
                              )
    run(args)



from typing import List

import numpy as np
import pandas as pd

from cenfind.core.data import Field
from cenfind.core.log import get_logger
from cenfind.core.structures import Centriole, Nucleus

logger = get_logger(__name__)


def frequency(df: pd.DataFrame) -> pd.DataFrame:
    """
    Count the absolute frequency of number of centriole per well or per field
    :param df: Df containing the number of centriole per nuclei
    :return: Df with absolute frequencies.

    Parameters
    ----------
    scored
    """
    cuts = [0, 1, 2, 3, 4, 5, np.inf]
    labels = "0 1 2 3 4 +".split(" ")

    result = pd.cut(df["score"], cuts, right=False, labels=labels,
                    include_lowest=True)
    result = result.groupby(["field", "channel"]).value_counts()
    result.name = "freq_abs"
    result = result.reset_index()
    result = result.rename({"score": "score_category"}, axis=1)
    result = result.pivot(index=["channel", "field"], columns="score_category")
    result.columns = labels

    return result


def proportion_cilia(field: Field, cilia: List[Centriole], nuclei: List[Nucleus], channel_cilia: int) -> pd.DataFrame:
    proportions = []
    ciliated = len(cilia) / len(nuclei)
    proportions.append({'field': field.name,
                        "channel_cilia": channel_cilia,
                        "n_nuclei": len(nuclei),
                        "n_cilia": len(cilia),
                        "p_ciliated": round(ciliated, 2)})
    return pd.DataFrame(proportions)



import json
from pathlib import Path
from typing import List

import numpy as np
import pandas as pd
import tifffile as tif

from cenfind.core.log import get_logger
from cenfind.core.structures import Centriole, Nucleus

logger = get_logger(__name__)


def save_assigned(dst: Path, assigned: np.ndarray) -> None:
    logger.info("Writing assigned matrix to %s" % str(dst))
    np.savetxt(str(dst), assigned, fmt='%i')


def save_assigned_centrioles(dst: Path, assigned_centrioles) -> None:
    result = pd.DataFrame(assigned_centrioles)
    result.columns = ['centriole_index', 'nucleus_index']
    result.to_csv(dst, sep='\t', index=False)


def save_points(dst: Path, centrioles: List[Centriole]) -> None:
    if len(centrioles) == 0:
        result = pd.DataFrame([])
        logger.info("No centriole detected")
    else:
        result = pd.DataFrame.from_dict({c.index: c.as_dict() for c in centrioles}, orient='index')
    result.to_csv(dst, index_label='index', index=False, sep='\t')


def save_contours(dst: Path, nuclei: List[Nucleus]) -> None:
    with open(dst, 'w') as file:
        json.dump({nucleus.index: nucleus.as_dict() for nucleus in nuclei}, file)
        logger.info('Writing contours to %s' % str(dst))


def save_visualisation(dst, vis: np.ndarray) -> None:
    logger.info("Writing visualisation to %s" % (str(dst)))
    tif.imwrite(dst, vis)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['cenfind',
 'cenfind.cli',
 'cenfind.core',
 'cenfind.publication',
 'cenfind.training']

package_data = \
{'': ['*']}

install_requires = \
['albumentations>=1.3.0,<2.0.0',
 'attrs>=22.2.0,<23.0.0',
 'csbdeep>=0.7.3,<0.8.0',
 'labelbox[data]>=3.46.0,<4.0.0',
 'llvmlite==0.39.1',
 'numba==0.56.4',
 'numpy>=1.23.5,<2.0.0',
 'opencv-python>=4.7.0.72,<5.0.0.0',
 'ortools==9.4.1874',
 'pandas>=1.4.1,<2.0.0',
 'protobuf==3.19.6',
 'python-dotenv>=0.21.1,<0.22.0',
 'pytomlpp>=1.0.10,<2.0.0',
 'scikit-image>=0.19.2,<0.20.0',
 'scikit-learn>=1.2.1,<2.0.0',
 'scipy>=1.7.1,<2.0.0',
 'spotipy-detector>=0.1.0,<0.2.0',
 'stardist>=0.8.3,<0.9.0',
 'tifffile>=2022.5.4,<2023.0.0',
 'tqdm>=4.62.3,<5.0.0']

extras_require = \
{':sys_platform == "darwin"': ['tensorflow-macos==2.9.0'],
 ':sys_platform == "win32" or sys_platform == "linux"': ['tensorflow==2.9.0']}

entry_points = \
{'console_scripts': ['cenfind = cenfind.__main__:main']}

setup_kwargs = {
    'name': 'cenfind',
    'version': '0.13.1',
    'description': 'Score cells for centrioles in IF data',
    'long_description': '# CenFind\n\nA command line interface to score cells for centrioles.\n\n## Introduction\n\n`cenfind` is a command line interface to detect and assign centrioles in immunofluorescence images of human cells.\nSpecifically, it orchestrates:\n\n- the detection of centrioles;\n- the detection of the nuclei;\n- the assignment of the centrioles to the nearest nucleus.\n\nYou can read more on it here: Bürgy, L., Weigert, M., Hatzopoulos, G. et al. CenFind: a deep-learning pipeline for efficient centriole detection in microscopy datasets. BMC Bioinformatics 24, 120 (2023). https://doi.org/10.1186/s12859-023-05214-2\n\n## Installation\n\n1. Install python via pyenv\n2. Download and set up 3.9.5 as local version\n3. Set up Python interpreter\n\n```shell\npyenv local 3.9.5\npyenv global 3.9.5\n```\n\n4. Create a virtual environment for CenFind\n\n```shell\npython -m venv venv-cenfind\nsource venv-cenfind/bin/activate\n```\n\n5. Check that `cenfind` is correctly installed by running:\n\n```shell\ncenfind --help\n```\n\n## Basic usage\n\n`cenfind` assumes a fixed folder structure.\nSpecifically, it expects the max-projection to be under the `projections` folder.\nEach file in projections is a z-max projected field of view (referred to as field, in the following) containing 4\nchannels (0, 1, 2, 3). The channel 0 usually contains the nuclei and the channels 1-3 contains centriolar markers.\n\n```text\n<project_name>/\n└── projections/\n```\n\n1. Run `score` with the arguments source, the index of the nuclei channel (usually 0 or 3), the channel to score and the\n   path to the model. You need to download it from https://figshare.com/articles/software/Cenfind_model_weights/21724421\n\n```shell\ncenfind score /path/to/dataset /path/to/model/ -n 0 -c 1 2 3\n```\n\n```shell\nusage: CENFIND score [-h] --channel_nuclei CHANNEL_NUCLEI [--channel_centrioles CHANNEL_CENTRIOLES [CHANNEL_CENTRIOLES ...]] [--channel_cilia CHANNEL_CILIA] [--vicinity VICINITY] [--cpu] dataset model\n\npositional arguments:\n  dataset               Path to the dataset\n  model                 Absolute path to the model folder\n\noptions:\n  -h, --help            show this help message and exit\n  --channel_nuclei CHANNEL_NUCLEI, -n CHANNEL_NUCLEI\n                        Channel index for nuclei segmentation, e.g., 0 or 3 (default: None)\n  --channel_centrioles CHANNEL_CENTRIOLES [CHANNEL_CENTRIOLES ...], -c CHANNEL_CENTRIOLES [CHANNEL_CENTRIOLES ...]\n                        Channel indices to analyse, e.g., 1 2 3 (default: [])\n  --channel_cilia CHANNEL_CILIA, -l CHANNEL_CILIA\n                        Channel indices to analyse cilium (default: None)\n  --vicinity VICINITY, -v VICINITY\n                        Distance threshold in pixel (default: 50 px) (default: 50)\n  --cpu                 Only use the cpu (default: False)\n```\n\n2. Check that the predictions are satisfactory by looking at the folders `visualisations/` and `statistics/`\n\n## The outputs in version 0.13.x\n\nIn version 0.13, we operated a shift in what cenfind-score outputs. Now, there are modular outputs that can be linked together depending on the applications. In the following section, each output is explained.\n\n### Assignment\n\nCenfind saves the assignment matrix in the assignment folder.\n\nThis matrix is NxC where the row indices correspond to nucleus ID and the column indices to the centriole ID. It describes which centrioles are assigned to which nucleus. One can compute the number of centrioles by cell by summing over the columns and to retrieve the nucleus ID of every centriole assigned by looking up the row number of the entry for a given centriole.\n\n### Centriole predictions\nCenfind saves a TSV file for each field of view with the detected centrioles and the channel used as well as the maximum intensity at the position.\n\n### Nuclei predictions\n\nCenfind saves a JSON file for each field of view with the detected nuclei. Each nucleus contour is saved as an entry in the JSON together with the channel index, the position (row, col) the summed intensity, the surface area, whether the nucleus is fully in the field of view.\n\n### Cilia\nIf specified by the user at the command line prompt, the cilia can be analysed in the given channel. In such cases, the folder called cilia will contain TSV files similar in structure to the one from centrioles.\n\n### Summary statistics\nThe statistics folder contains precomputed information about the distribution of centriole number (statistics.tsv), TSV files for pairs of assigned centrioles their nucleus if possible. If the cilia are analysed, a TSV file containing the fraction of ciliated cells is saved as well.\n\n## Running `cenfind score` in the background\n\nWhen you exit the shell, running programs receive the SIGHUP, which aborts them. This is undesirable if you need to\nclose your shell for some reason. Fortunately, you can make your program ignore this signal by prepending the program\nwith the `nohup` command. Moreover, if you want to run your program in the background, you can append the ampersand `&`.\nIn practice, run `nohup cenfind score ... &` instead of `cenfind score ...`.\n\nThe output will be written to the file `nohup.out` and you can peek the progress by running `tail -F nohup.out`, the\nflag `-F` will refresh the screen as the file is being written. Enter Ctrl-C to exit the tail program.\n\nIf you want to kill the program score, run  `jobs` and then run `kill <jobid>`. If you see no jobs, check the\nlog `nohup.out`; it can be done or the program may have crashed, and you can check the error there.\n\n## Evaluating the quality of the model on a new dataset\n\nThe initial model M is fitted using a set of five representative datasets, hereafter referred to as the standard\ndatasets (DS1-5).\nIf your type of data deviates too much from the standard dataset, M may perform less well.\n\nSpecifically, when setting out to score a new dataset, you may be faced with one of three situations, as reflected by\nthe corresponding F1 score (i.e., 2TP/2TP+FN+FP, TP: true positive, FP: false positive; FN: false negative):\n(1) the initial model (M) performs well on the new dataset (0.9 ≤ F1 ≤ 1); in this case, model M is used;\n(2) model M performs significantly worse on the new dataset (0.5 ≤ F1 < 0.9); in this case, you may want to consider\nretraining the model (see below);\n(3) the model does not work at all (0 ≤ F1 < 0.5); such a low F1-value probably means that the features of the data set\nare too distant from the original representative data set to warrant retraining starting from M.\n\nBefore retraining a model (2), verify once more the quality of the data, which needs to be sufficiently good in terms of\nsignal over noise to enable efficient learning.\nIf this is not the case, it is evident that the model will not be able to learn well.\nIf you, as a human being, cannot tell the difference between a real focus and a stray spot using a single channel at\nhand (i.e., not looking at other channels), the same will hold for the model.\n\nTo retrain the model, you first must annotate the dataset, divide it randomly into training and test sets (90 % versus 10 % of the data, respectively).\nNext, the model is trained with the 90 % set, thus generating a new model, M*.\nLast, you will evaluate the gain of performance on the new dataset, as well as the potential loss of performance on the standard datasets.\n\n### Detailed training procedure:\n\n1. Split the dataset into training (90%) and test (10%) sets, each containing one field of view and the channel to use.\n   This helps trace back issues during the training and renders the model fitting reproducible.\n\n2. Label all the images present in training and test sets using Labelbox. To upload the images, please create the vignettes first and then upload them once you have a project set up.\n3. Save all foci coordinates (x, y), origin at top-left, present in one field of view as one text file under\n   /path/to/dataset/annotation/centrioles/ with the naming scheme <dataset_name>_max_C<channel_index>.txt.\n4. Evaluate the newly annotated dataset using the model M by computing the F1 score.\n5. If the performance is poor (i.e., F1 score < 0.9), fit a new model instance, M*, with the standard dataset plus the\n   new dataset (90% in each case).\n6. Test performance of model M* on the new data set; hopefully the F1 score will now be ≥ 0.9 (if not: consider\n   increasing size of annotated data).\n7. Test performance of model M* on the standard datasets; if performance of F1* ≥ F1, then save M* as the new M (\n   otherwise keep M* as a separate model for the new type of data set).\n',
    'author': 'Leo Burgy',
    'author_email': 'leo.burgy@epfl.ch',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/UPGON/cenfind',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



import pandas as pd
from matplotlib import pyplot as plt

from cenfind.publication.constants import ROOT_DIR
from cenfind.publication.constants import protein_positions

pd.options.display.max_columns = 50


def main():
    path_data = ROOT_DIR / "out/performances_20221006_130126.csv"
    data = pd.read_csv(path_data)
    data = data.loc[data["n_preds"] > 0]

    datasets_unique = data.dataset.unique()

    fig, axs = plt.subplots(
        1, len(datasets_unique), figsize=(5 * len(datasets_unique), 5)
    )
    for r, dataset in enumerate(datasets_unique):
        sub = data.loc[data["dataset"] == dataset]
        ax = axs[r]
        for channel in sub.channel.unique():
            marker = protein_positions[dataset][channel]

            ssub = sub.loc[sub["channel"] == channel]
            ssub = ssub.sort_values(by="threshold")
            ax.plot(ssub["recall"], ssub["precision"], ls="-", label=marker)

        ax.spines.right.set_visible(False)
        ax.spines.top.set_visible(False)
        ax.set_xlim(0, 1)
        ax.set_ylim(0, 1.1)
        ax.set_xlabel("recall")
        ax.set_ylabel("precision")
        ax.set_title(dataset)
        ax.legend()
    fig.savefig(ROOT_DIR / f"out/checks/precision_recall.png")


if __name__ == "__main__":
    main()



import re
from pathlib import Path

import pandas as pd

PREFIX_LOCAL = Path("/Users/buergy/Dropbox/epfl/datasets")
PREFIX_REMOTE = Path("/data1/centrioles/canonical")

ROOT_DIR = Path("/home/buergy/projects/cenfind")
datasets = [
    "RPE1wt_CEP63+CETN2+PCNT_1",
    "RPE1wt_CP110+GTU88+PCNT_2",
    "RPE1wt_CEP152+GTU88+PCNT_1",
    "U2OS_CEP63+SAS6+PCNT_1",
    "RPE1p53+Cnone_CEP63+CETN2+PCNT_1",
]

protein_positions = {
    "RPE1wt_CEP63+CETN2+PCNT_1": ("DAPI", "CEP63", "CETN2", "PCNT"),
    "RPE1wt_CP110+GTU88+PCNT_2": ("DAPI", "CP110", "GTU88", "PCNT"),
    "RPE1wt_CEP152+GTU88+PCNT_1": ("DAPI", "CEP152", "GTU88", "PCNT"),
    "U2OS_CEP63+SAS6+PCNT_1": ("DAPI", "CEP63", "SAS6", "PCNT"),
    "RPE1p53+Cnone_CEP63+CETN2+PCNT_1": ("DAPI", "CEP63", "CETN2", "PCNT"),
    "denovo": ("CEP63", "CEP135", "DAPI"),
}

protein_names = {
    "CEP63": "Cep63",
    "CEP152": "Cep152",
    "CEP135": "Cep135",
    "GTU88": "γ-Tubulin",
    "PCNT": "Pericentrin",
    "CETN2": "Centrin",
    "SAS6": "SAS6",
    "CP110": "Cp110",
    "AcTub": "Ac. Tub",
    "Plk4": "Plk4",
    "STIL": "STIL",
}

celltype_names = {
    "RPE1wt": "RPE-1 WT",
    "RPE1p53": "RPE-1 p53-/-",
    "U2OS": "U2-OS",
    "U2OSwt": "U2-OS",
}

pattern_dataset = re.compile(
    "(?P<cell_type>[a-zA-Z0-9.-]+)([+_](?P<treatment>\w+))?_(?P<markers>[\w+]+)_(?P<replicate>\d)"
)

UNITS = {"well", "field"}

if __name__ == "__main__":
    datasets = pd.DataFrame(datasets)
    cell_types = pd.DataFrame().from_dict(celltype_names, orient="index")
    protein_positions = pd.DataFrame().from_dict(protein_positions, orient="index")
    proteins_names = pd.DataFrame.from_dict(protein_names, orient="index")

    datasets.to_csv("data/datasets.tsv", sep="\t")
    protein_positions.to_csv("data/protein_positions.tsv", sep="\t")
    proteins_names.to_csv("data/proteins_names.tsv", sep="\t")
    cell_types.to_csv("data/cell_types.tsv", sep="\t")



import argparse
from pathlib import Path

import cv2
from tqdm import tqdm

from cenfind.core.data import Dataset
from cenfind.core.visualisation import create_vignette


def register_parser(parent_subparsers):
    parser = parent_subparsers.add_parser(
        "vignettes",
        help="VIGNETTE: create png version of channel+nuclei for annotation tool",
    )
    parser.add_argument("dataset", type=Path, help="the path to the dataset")
    parser.add_argument(
        "--channel_nuclei",
        "-n",
        type=int,
        required=True,
        help="the index of the nuclei (often, 0 or 3, first or last)",
    )
    parser.add_argument(
        "--channel_centrioles",
        "-c",
        type=int,
        required=True,
        nargs="+",
        help="the index of the channels",
    )
    parser.add_argument(
        "--projection_suffix",
        "-s",
        type=str,
        default="",
        help="the suffix indicating projection, e.g., `_max` or `_Projected`, empty if not specified",
    )

    return parser


def run(args):
    dataset = Dataset(args.dataset, projection_suffix=args.projection_suffix)

    pbar = tqdm(dataset.fields)
    for field in pbar:
        for channel_id in args.channel_centrioles:
            pbar.set_description(f"{field.name}: {channel_id}")
            vignette = create_vignette(field, channel_id, args.channel_nuclei)
            dst = (
                    dataset.vignettes
                    / f"{field.name}{args.projection_suffix}_C{channel_id}.png"
            )
            cv2.imwrite(str(dst), vignette)


if __name__ == "__main__":
    args = argparse.Namespace(dataset=Path('data/dataset_test'),
                              model=Path('models/master'),
                              channel_nuclei=0,
                              channel_centrioles=[1, 2],
                              projection_suffix='_max')
    run(args)
